@@ -41,6 +41,10 @@
 # define __NR_inotify_init		318
 # define __NR_inotify_add_watch		319
 # define __NR_inotify_rm_watch		320
+#elif defined (__hppa__)
+# define __NR_inotify_init		269
+# define __NR_inotify_add_watch		270
+# define __NR_inotify_rm_watch		271
 #elif defined (__mips__)
 # define _MIPS_SIM_ABI32		1
 # define _MIPS_SIM_NABI32		2
@@ -72,6 +76,10 @@
 # define __NR_inotify_init		(__NR_Linux + 269)
 # define __NR_inotify_add_watch		(__NR_Linux + 270)
 # define __NR_inotify_rm_watch		(__NR_Linux + 271)
+#elif defined (__mc68000__)
+# define __NR_inotify_init		284
+# define __NR_inotify_add_watch		285
+# define __NR_inotify_rm_watch		286
 #else
 # error "Unsupported architecture!"
 #endif
