@@ -3,9 +3,9 @@
 # pyinotify.py - high level python interface to inotify
 # Copyright (C) 2005-2007 Sbastien Martini <sebastien.martini@gmail.com>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,8 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
 """
 Interface for inotify.
@@ -153,36 +152,39 @@ class EventsCodes:
     @cvar ALL_EVENTS: Alias for considering all of the events.
     @type ALL_EVENTS: int
     """
-    IN_ACCESS = 0x00000001  # File was accessed
-    IN_MODIFY = 0x00000002  # File was modified
-    IN_ATTRIB = 0x00000004  # Metadata changed
-    IN_CLOSE_WRITE = 0x00000008  # Writtable file was closed
-    IN_CLOSE_NOWRITE = 0x00000010  # Unwrittable file closed
-    IN_OPEN = 0x00000020  # File was opened
-    IN_MOVED_FROM = 0x00000040  # File was moved from X
-    IN_MOVED_TO = 0x00000080  # File was moved to Y
-    IN_CREATE = 0x00000100  # Subfile was created
-    IN_DELETE = 0x00000200  # Subfile was deleted
-    IN_DELETE_SELF = 0x00000400  # Self (watched item itself) was deleted
-    IN_MOVE_SELF = 0x00000800  # Self (watched item itself) was moved
-
-    IN_UNMOUNT = 0x00002000  # Backing fs was unmounted
-    IN_Q_OVERFLOW = 0x00004000  # Event queued overflowed
-    IN_IGNORED = 0x00008000  # File was ignored
-
-    # special flags
-    IN_ONLYDIR = 0x01000000  # only watch the path if it is a directory
-    IN_DONT_FOLLOW = 0x02000000  # don't follow a symlink
-    IN_MASK_ADD = 0x20000000  # add to the mask of an already existing watch
-    IN_ISDIR = 0x40000000  # event occurred against dir
-    IN_ONESHOT = 0x80000000  # only send event once
-
-    # all events
-    ALL_EVENTS = IN_ACCESS | IN_MODIFY | IN_ATTRIB | IN_CLOSE_WRITE | \
-                 IN_CLOSE_NOWRITE | IN_OPEN | IN_MOVED_FROM | \
-                 IN_MOVED_TO | IN_DELETE | IN_CREATE | IN_DELETE_SELF | \
-                 IN_MOVE_SELF
 
+    # The idea here is 'configuration-as-code' - this way, we get our nice class
+    # constants, but we also get nice human-friendly text mappings to do lookups
+    # against as well, for free:
+    FLAG_COLLECTIONS = {'OP_FLAGS': {
+        'IN_ACCESS'        : 0x00000001,  # File was accessed
+        'IN_MODIFY'        : 0x00000002,  # File was modified
+        'IN_ATTRIB'        : 0x00000004,  # Metadata changed
+        'IN_CLOSE_WRITE'   : 0x00000008,  # Writable file was closed
+        'IN_CLOSE_NOWRITE' : 0x00000010,  # Unwritable file closed
+        'IN_OPEN'          : 0x00000020,  # File was opened
+        'IN_MOVED_FROM'    : 0x00000040,  # File was moved from X
+        'IN_MOVED_TO'      : 0x00000080,  # File was moved to Y
+        'IN_CREATE'        : 0x00000100,  # Subfile was created
+        'IN_DELETE'        : 0x00000200,  # Subfile was deleted
+        'IN_DELETE_SELF'   : 0x00000400,  # Self (watched item itself) was deleted
+        'IN_MOVE_SELF'     : 0x00000800,  # Self (watched item itself) was moved
+        },
+                        'EVENT_FLAGS': {
+        'IN_UNMOUNT'       : 0x00002000,  # Backing fs was unmounted
+        'IN_Q_OVERFLOW'    : 0x00004000,  # Event queued overflowed
+        'IN_IGNORED'       : 0x00008000,  # File was ignored
+        },
+                        'SPECIAL_FLAGS': {
+        'IN_ONLYDIR'       : 0x01000000,  # only watch the path if it is a directory
+        'IN_DONT_FOLLOW'   : 0x02000000,  # don't follow a symlink
+        'IN_MASK_ADD'      : 0x20000000,  # add to the mask of an already existing watch
+        'IN_ISDIR'         : 0x40000000,  # event occurred against dir
+        'IN_ONESHOT'       : 0x80000000,  # only send event once
+        },
+                        }
+    
+    
     def _get_event_name(mask):
         """
         Return the event name associated to the mask parameter, never
@@ -194,14 +196,33 @@ class EventsCodes:
         @return: event name.
         @rtype: str or None
         """
-        for ec in dir(EventsCodes):
-            if ec.startswith('IN_') and ec != 'IN_ISDIR' and \
-                   (getattr(EventsCodes, ec) & mask):
-                return ec
-
+        return reduce(lambda x, y: x + '|' + y,
+                      [ec for ec, val in EventsCodes.ALL_FLAGS.iteritems() \
+                       if val & mask])
+    
     _get_event_name = staticmethod(_get_event_name)
 
 
+# So let's now turn the configuration into code
+EventsCodes.ALL_FLAGS = {}
+for flag, fval in EventsCodes.FLAG_COLLECTIONS.iteritems():
+    # Make the collections' members directly accessible through the
+    # class dictionary
+    setattr(EventsCodes, flag, fval)
+    
+    # Collect all the flags under a common umbrella
+    EventsCodes.ALL_FLAGS.update(fval)
+    
+    # Make the individual masks accessible as 'constants' via class
+    # attributes
+    for mask, mval in fval.iteritems():
+        setattr(EventsCodes, mask, mval)
+
+# all 'normal' events
+EventsCodes.ALL_EVENTS = reduce(lambda x, y: x | y,
+                                EventsCodes.OP_FLAGS.itervalues())
+
+
 class _Event:
     """
     Event structure, represent events raised by the system. This
@@ -227,7 +248,7 @@ class _Event:
 class _RawEvent(_Event):
     """
     Raw event, it contains only the informations provided by the system.
-    It doesn't infers nothing.
+    It doesn't infer anything.
     """
     def __init__(self, wd, mask, cookie, name):
         """
@@ -306,9 +327,8 @@ class _ProcessEvent:
                                   unknown event.
         """
         mask = event.mask
-        for ec in dir(EventsCodes):
-            if ec.startswith('IN_') and ec != 'IN_ISDIR' and \
-                   (mask & getattr(EventsCodes, ec)):
+        for ec in EventsCodes.ALL_FLAGS.iterkeys():
+            if ec != 'IN_ISDIR' and (mask & EventsCodes.ALL_FLAGS[ec]):
                 # 1- look for process_EVENT_NAME
                 meth = getattr(self, 'process_%s' % ec, None)
                 if meth is not None:
@@ -430,6 +450,7 @@ class _SysProcessEvent(_ProcessEvent):
         Only signal overflow, most of the common flags are irrelevant
         for this event (path, wd, name).
         """
+        #TODO: Option to automatically make the Q bigger at this point?
         return Event({'mask': raw_event.mask,
                       'event_name':EventsCodes._get_event_name(raw_event.mask)})
 
@@ -1040,6 +1061,9 @@ def command_line():
     parser.add_option("-a", "--auto_add", action="store_true",
                       dest="auto_add",
                       help="Automatically add watches on new directories")
+    parser.add_option("-e", "--events-list", metavar="EVENT[,...]",
+                      dest="events_list",
+                      help="A comma-separated list of events to watch for - see the man page for valid options (defaults to everything)")
 
     (options, args) = parser.parse_args()
 
@@ -1057,8 +1081,19 @@ def command_line():
     # notifier instance and init
     notifier = Notifier(wm)
 
-    # watch path for all events
-    wm.add_watch(path, EventsCodes.ALL_EVENTS,
+    # What mask to apply
+    mask = 0
+    if options.events_list:
+        events_list = options.events_list.split(',')
+        for ev in events_list:
+            evcode = EventsCodes.ALL_FLAGS.get(ev, 0)
+            if evcode:
+                mask |= evcode
+            else:
+                parser.error("The event '%s' specified with option -e is not valid" % ev)
+    else:
+        mask = EventsCodes.ALL_EVENTS
+    wm.add_watch(path, mask,
                  rec=options.recursive, auto_add=options.auto_add)
 
     print 'start monitoring %s, (press c^c to halt pyinotify)' % path
